/*
  unicode-jp.c -- JIS X 0208 <=> UCS-2 (UTF-8) converter
  written by N. Tsuchimura
*/

#ifndef KPATHSEA_UNICODE_JP_H
#define KPATHSEA_UNICODE_JP_H

#include <kpathsea/c-proto.h>
#include <kpathsea/types.h>

#ifdef DEFAULT_KANJI

/* convert a JIS X 0208 char to UCS-2 */
extern KPSEDLL int JIStoUCS2 P1H(int);

/* convert a UCS-2 char to JIS X 0208 */
extern KPSEDLL int UCS2toJIS P1H(int);

/* UTF-8 -> UCS-2 */
#define UTF8BtoUCS2(a,b)   ((((a)&0x1f) << 6) | ((b)&0x3f))
#define UTF8CtoUCS2(a,b,c) ((((a)&0x0f) <<12) | (((b)&0x3f) << 6) | ((c)&0x3f))
#define UTF8DtoUCS2(a,b,c,d) \
  ((((a)&0x07) <<18) | (((b)&0x3f) <<12) | (((c)&0x3f) << 6) | ((d)&0x3f))

/* UCS-2 -> UTF-8 */
#define UCS2toUTF8B1(x)  (0xc0 + (((x) >>  6) & 0x1f))
#define UCS2toUTF8B2(x)  (0x80 + (((x)      ) & 0x3f))

#define UCS2toUTF8C1(x)  (0xe0 + (((x) >> 12) & 0x0f))
#define UCS2toUTF8C2(x)  (0x80 + (((x) >>  6) & 0x3f))
#define UCS2toUTF8C3(x)  (0x80 + (((x)      ) & 0x3f))

#define UCS2toUTF8D1(x)  (0xf0 + (((x) >> 18) & 0x07))
#define UCS2toUTF8D2(x)  (0x80 + (((x) >> 12) & 0x3f))
#define UCS2toUTF8D3(x)  (0x80 + (((x) >>  6) & 0x3f))
#define UCS2toUTF8D4(x)  (0x80 + (((x)      ) & 0x3f))

/* for U+3099 or  U+309A */
extern KPSEDLL int get_voiced_sound P2H(int, boolean);

#define U_BOM           0xFEFF
#define U_VOICED        0x3099
#define U_SEMI_VOICED   0x309A

#endif /* DEFAULT_KANJI */
#endif /* KPATHSEA_UNICODE_JP_H */
